use std::net::IpAddr;

use url::Url;
use serde::{Deserialize};
use config::{Config, ConfigError, Environment, File};

#[derive(Debug, Deserialize, Clone)]
pub struct PowerDnsOidcTsigkeyConfig {
    /// OIDC Provider
    pub oidc: OidcConfig,
    /// PowerDNS config
    pub powerdns: PowerDnsConfig,
    /// Logging config
    pub log: LogConfig,
    /// HTTP(s) config
    pub server: ServerConfig,
}

#[derive(Debug, Deserialize, Clone)]
pub struct PowerDnsConfig {
    /// URL where PowerDNS API can be reached
    pub url: Url,
    /// Api Token to use for authentication
    pub api_token: String,
}

#[derive(Debug, Deserialize, Clone)]
pub struct LogConfig {
    /// The log level
    pub level: String,
}

#[derive(Debug, Deserialize, Clone)]
pub struct ServerConfig {
    /// IpAddress to listen on
    pub bind_address: IpAddr,
    /// Port to listen on
    pub port: u16,
    /// TLS config
    pub tls: ServerTlsConfig,
}

#[derive(Debug, Deserialize, Clone)]
pub struct ServerTlsConfig {

}

#[derive(Debug, Deserialize, Clone)]
pub struct OidcConfig {
    pub issuer: Url,
    pub client_id: String,
    pub client_secret: String,
    pub scopes: Vec<String>,
    pub username_claim: String,
    pub validation: OidcValidationConfig,
}

#[derive(Debug, Deserialize, Clone)]
pub struct OidcValidationConfig {
    pub issuer: Vec<Url>,
    pub audience: Vec<String>,
    pub not_before: bool,
    pub expiry: bool,
}

impl PowerDnsOidcTsigkeyConfig {
    pub fn load(filename: &str) -> Result<Self, ConfigError> {
        Config::builder()
            .add_source(File::with_name(filename))
            .add_source(Environment::with_prefix("POTK").separator("_"))
            .set_default("log.level", "INFO")?
            .set_default("server.bind_address", "::")?
            .set_default("server.port", 8080)?
            .build()?
            .try_deserialize()
    }
}
